/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2013 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

/**
\addtogroup RxCore
**/
/*@{*/

// ///////////////////////////////////////////////////
// Image Type from Core version 2.8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Image data types. </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_BYTE                             0x1400
#define RX_IMG_UNSIGNED_BYTE    0x1401
#define RX_IMG_SHORT                    0x1402
#define RX_IMG_UNSIGNED_SHORT   0x1403
#define RX_IMG_INT                              0x1404
#define RX_IMG_UNSIGNED_INT             0x1405
#define RX_IMG_FLOAT                    0x1406
#define RX_IMG_DOUBLE                   0x140A

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Pixel color types. </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_RGB              0x1907
#define RX_IMG_RGBA             0x1908
#define RX_IMG_BGR              0x80E0
#define RX_IMG_BGRA             0x80E1
#define RX_IMG_LUMINANCE        0x1909
#define RX_IMG_LUMINANCE_ALPHA  0x190A

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Pixel color bayer types </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_BAYER_RG                 0x2001
#define RX_IMG_BAYER_BG                 0x2002
#define RX_IMG_BAYER_GR                 0x2003
#define RX_IMG_BAYER_GB                 0x2004

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// namespace: Rx.Interop.Runtime28
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
namespace Rx
{
	namespace Interop
	{
		namespace Runtime28
		{
			/**
			\addtogroup RxCore_Image
			**/
			/*@{*/

			namespace EDataType
			{
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				/// <summary>
				/// 	The image data types.
				/// </summary>
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				enum ID
				{
					Void   = 0,
					Byte   = RX_IMG_BYTE,
					UByte  = RX_IMG_UNSIGNED_BYTE,
					Short  = RX_IMG_SHORT,
					UShort = RX_IMG_UNSIGNED_SHORT,
					Int    = RX_IMG_INT,
					UInt   = RX_IMG_UNSIGNED_INT,
					Float  = RX_IMG_FLOAT,
					Double = RX_IMG_DOUBLE
				};
			}

			namespace EPixelType
			{
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				/// <summary>
				/// 	The image pixel types..
				/// </summary>
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				enum ID
				{
					None = 0,
					/// 3 Channels per pixel (Red, Green, Blue)
					RGB = RX_IMG_RGB,
					/// 4 Channels per pixel (Red, Green, Blue, Alpha)
					RGBA = RX_IMG_RGBA,
					/// 3 Channels per pixel (Blue, Green, Red)
					BGR = RX_IMG_BGR,
					/// 4 Channels per pixel (Blue, Green, Red, Alpha)
					BGRA = RX_IMG_BGRA,
					/// 1 Channel per pixel (Luminance)
					Lum = RX_IMG_LUMINANCE,
					/// 2 Channels per pixel (Luminance, Alpha)
					LumA = RX_IMG_LUMINANCE_ALPHA,
					/// 1 Channel per pixel (Bayer pattern RGGB)
					BayerBG = RX_IMG_BAYER_BG,
					/// 1 Channel per pixel (Bayer pattern BGGR)
					BayerGB = RX_IMG_BAYER_GB,
					/// 1 Channel per pixel (Bayer pattern GRBG)
					BayerGR = RX_IMG_BAYER_GR,
					/// 1 Channel per pixel (Bayer pattern GBRG)
					BayerRG = RX_IMG_BAYER_RG
				};
			}
		}
	}
}

/*@}*/
