/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// 	Defines the different video modes.
/// 	The enum order is: "video description"_"zoom level"_"bit depth"
///
/// 	i.e.:
/// 	HD_ZOOM1_BIT8 would produce a output resolution of 1920x1080 pixel. ZOOM1 indicates that nearly the whole image
/// 	of the sensor will be binned by factor 4 to get this resolution. 8 bit is the bit depth of each pixel.
///
/// 	HD_ZOOM4_BIT8 would produce a output resolution of 1920x1080 pixel. ZOOM4 indicates that the sensor will crop
/// 	1920x1080 out of the full size 7716x5364 resolution. 8 bit is the bit depth of each pixel.
/// </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef RX_MANAGED_ENUM
	public enum class ECamVideoMode
#else
	namespace ECamVideoMode
	{
		enum ID
#endif
{
	/// <summary> None </summary>
	None = 0,

	// full resolution

	FULLRES_ZOOM1_BIT8,							// final resolution: 7716x5364
	FULLRES_ZOOM1_BIT10,						//
	FULLRES_ZOOM1_BIT12,						//

	// common 16:9 modes

	FUHD_ZOOM1_BIT8,							// final resolution: 7680x4320
	FUHD_ZOOM1_BIT10,							//
	FUHD_ZOOM1_BIT12,							//

	UHD_ZOOM1_BIT8,								// final resolution: 3840x2160
	UHD_ZOOM1_BIT10,							//
	UHD_ZOOM1_BIT12,							//
	UHD_ZOOM2_BIT8,								//
	UHD_ZOOM2_BIT10,							//
	UHD_ZOOM2_BIT12,							//

	HD_ZOOM1_BIT8,								// final resolution: 1920x1080
	HD_ZOOM1_BIT10,								//
	HD_ZOOM1_BIT12,								//
	HD_ZOOM2_BIT8,								//
	HD_ZOOM2_BIT10,								//
	HD_ZOOM2_BIT12,								//
	HD_ZOOM4_BIT8,								//
	HD_ZOOM4_BIT10,								//
	HD_ZOOM4_BIT12,								//

	qHD_ZOOM1_BIT8,								// final resolution: 960x540
	qHD_ZOOM1_BIT10,							//
	qHD_ZOOM1_BIT12,							//
	qHD_ZOOM2_BIT8,								//
	qHD_ZOOM2_BIT10,							//
	qHD_ZOOM2_BIT12,							//
	qHD_ZOOM4_BIT8,								//
	qHD_ZOOM4_BIT10,							//
	qHD_ZOOM4_BIT12,							//
	qHD_ZOOM8_BIT8,								//
	qHD_ZOOM8_BIT10,							//
	qHD_ZOOM8_BIT12,							//

	// square 1:1 modes

	SQUARE5280_ZOOM1_BIT8,						// final resolution: 5280x5280
	SQUARE5280_ZOOM1_BIT10,
	SQUARE5280_ZOOM1_BIT12,

	SQUARE2640_ZOOM1_BIT8,						// final resolution: 2640x2640
	SQUARE2640_ZOOM1_BIT10,						//
	SQUARE2640_ZOOM1_BIT12,						//
	SQUARE2640_ZOOM2_BIT8,						//
	SQUARE2640_ZOOM2_BIT10,						//
	SQUARE2640_ZOOM2_BIT12,						//

	SQUARE1320_ZOOM1_BIT8,						// final resolution: 1320x1320
	SQUARE1320_ZOOM1_BIT10,						//
	SQUARE1320_ZOOM1_BIT12,						//
	SQUARE1320_ZOOM2_BIT8,						//
	SQUARE1320_ZOOM2_BIT10,						//
	SQUARE1320_ZOOM2_BIT12,						//
	SQUARE1320_ZOOM4_BIT8,						//
	SQUARE1320_ZOOM4_BIT10,						//
	SQUARE1320_ZOOM4_BIT12,						//

	SQUARE660_ZOOM1_BIT8,						// final resolution: 660x660
	SQUARE660_ZOOM1_BIT10,						//
	SQUARE660_ZOOM1_BIT12,						//
	SQUARE660_ZOOM2_BIT8,						//
	SQUARE660_ZOOM2_BIT10,						//
	SQUARE660_ZOOM2_BIT12,						//
	SQUARE660_ZOOM4_BIT8,						//
	SQUARE660_ZOOM4_BIT10,						//
	SQUARE660_ZOOM4_BIT12,						//
	SQUARE660_ZOOM8_BIT8,						//
	SQUARE660_ZOOM8_BIT10,						//
	SQUARE660_ZOOM8_BIT12,						//
};
#ifndef RX_MANAGED_ENUM
	}	// Namespace
#endif
